/*
 * Copyright 2013 Palantir Technologies, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

/// <reference path='ruleWalker.ts'/>

module Lint {

    export interface RuleWalkerState {
        position: number;
        token: TypeScript.ISyntaxToken;
    }

    export class StateAwareRuleWalker extends RuleWalker {
        private lastState: RuleWalkerState;

        public visitToken(token: TypeScript.ISyntaxToken): void {
            // Skip compiler insertions of empty tokens
            if (token.value() !== null) {
                this.lastState = {
                    position: this.position() + token.leadingTriviaWidth(),
                    token: token
                };
            }

            super.visitToken(token);
        }

        public getLastState(): RuleWalkerState {
            return this.lastState;
        }
    }

}
