/*
 * Copyright 2013 Palantir Technologies, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

/// <reference path='rule.ts'/>
/// <reference path='abstractRule.ts'/>

module Lint.Rules {

    export class DebugRule extends AbstractRule {

        constructor() {
            super("debug");
        }

        public isEnabled() : boolean {
            return this.getValue() === true;
        }

        public apply(syntaxTree: TypeScript.SyntaxTree): RuleFailure[] {
            return this.applyWithWalker(new DebugWalker(syntaxTree));
        }
    }

    class DebugWalker extends Lint.RuleWalker {
        static DEBUG_FAILURE = "use of debugger statements is disallowed";

        public visitToken(token : TypeScript.ISyntaxToken): void {
            this.handleToken(token);
            super.visitToken(token);
       }

        private handleToken(operatorToken: TypeScript.ISyntaxToken) {
            var failure = null;
            var operatorKind = operatorToken.kind();

            if (operatorKind === TypeScript.SyntaxKind.DebuggerKeyword) {
                this.addFailure(this.createFailure(this.position(), DebugWalker.DEBUG_FAILURE));
            }
        }
    }

}
